package com.example.pmulabs.designElem.items

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.requiredHeight
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextDecoration
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.example.pmulabs.room.database.NewsPortalDatabase
import com.example.pmulabs.room.models.Comment
import com.example.pmulabs.room.models.User
import com.example.pmulabs.viewModels.SharedViewModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CommentItem(modifier: Modifier = Modifier,comm : Comment,sharedViewModel: SharedViewModel) {
    val context = LocalContext.current

    var user by remember { mutableStateOf<User?>(null) }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).userDao()
                .getUserById(comm.userId).collect { data ->
                    user=data
                }
        }
    }

    val argument = sharedViewModel.argument.value
    var getUser by remember { mutableStateOf<User?>(null) }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).userDao()
                .getUserById(argument.toString().toInt()).collect { data ->
                    getUser=data
                }
        }
    }

    var openDialog by remember { mutableStateOf(false) }
    var text by remember { mutableStateOf(comm.text) }

    if (openDialog) {
        AlertDialog(
            onDismissRequest = {
                openDialog = false
                text=comm.text
            },
            content = {
                Column(
                    modifier = Modifier
                        .clip(RoundedCornerShape(16.dp))
                        .background(Color.White)
                        .padding(16.dp),
                ) {
                    Text(
                        text = "Edit your comment",
                        color = Color(0xff423a99),
                        style = TextStyle(
                            fontSize = 20.sp,
                        ),
                    )
                    OutlinedTextField(
                        value = text,
                        onValueChange = {
                            text=it
                        },
                        colors = TextFieldDefaults.textFieldColors(
                            containerColor = Color.White,
                            focusedIndicatorColor = Color(0xff423a99),
                            focusedLabelColor = Color(0xff423a99),
                            unfocusedIndicatorColor = Color(0xff423a99),
                            unfocusedLabelColor = Color(0xff423a99)
                        ),
                        modifier = Modifier
                            .fillMaxWidth()
                            .wrapContentHeight()
                            .padding(10.dp),
                        label = { Text("Comment") },
                        placeholder = { Text("Write comment...") },
                        minLines = 3,
                        maxLines = 3,
                    )
                    Row(
                        modifier = Modifier.padding(all = 8.dp),
                        horizontalArrangement = Arrangement.Center
                    ) {
                        Button(
                            colors = ButtonColors(
                                containerColor = Color(0xff423a99),
                                disabledContainerColor = Color(0xff423a99),
                                contentColor = Color.White,
                                disabledContentColor = Color.White
                            ),
                            onClick = {
                                openDialog = false
                                comm.text=text
                                CoroutineScope(Dispatchers.IO).launch {
                                    NewsPortalDatabase.getInstance(context).commentDao().update(comm)
                                }
                            },
                            modifier = Modifier
                                .padding(start=100.dp)
                                .fillMaxWidth(0.5f)
                                .height(40.dp)
                        ) {
                            Text(text = "Edit", fontSize = 20.sp)
                        }
                    }
                }
            },
        )
    }

    Row(
        modifier = Modifier
            .padding(3.dp)
            .fillMaxWidth()
            .background(Color.White)
    )
    {
        Column(
            modifier= Modifier
                .padding(start = 10.dp,top=10.dp)
        ) {
            Row() {
                Text(
                    text = "${user?.nickname}",
                    color = Color(0xff423a99),
                    style = TextStyle(
                        textDecoration = TextDecoration.Underline,
                        fontSize = 17.sp,
                        fontWeight = FontWeight.Bold
                    ),
                )

                if(getUser?.id==comm.userId) {
                    Icon(
                        modifier=Modifier
                            .requiredHeight(20.dp)
                            .clickable {
                                openDialog=true
                            },
                        imageVector = Icons.Filled.Edit,
                        contentDescription = "Update Icon",
                        tint = Color(0xff423a99)
                    )
                    Icon(
                        modifier=Modifier
                            .requiredHeight(20.dp)
                            .clickable {
                                CoroutineScope(Dispatchers.IO).launch {
                                    NewsPortalDatabase.getInstance(context).commentDao().delete(comm)
                                }
                            },
                        imageVector = Icons.Filled.Close,
                        contentDescription = "Delete Icon",
                        tint = Color(0xff423a99)
                    )
                }
            }
            Text(
                text = "${comm.text}",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 15.sp, fontWeight = FontWeight.Bold),
            )
            Spacer(modifier = Modifier.padding(10.dp))
        }
    }

}
