package com.example.pmulabs.screensMobile

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.requiredHeight
import androidx.compose.foundation.layout.requiredWidth
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import com.example.pmulabs.basecomponents.navigate.ARTICLE_ARGUMENT_KEY
import com.example.pmulabs.designElem.elem.BackButton
import com.example.pmulabs.designElem.items.CommentItem
import com.example.pmulabs.room.database.NewsPortalDatabase
import com.example.pmulabs.room.models.Article
import com.example.pmulabs.room.models.Comment
import com.example.pmulabs.room.models.Tag
import com.example.pmulabs.room.models.User
import com.example.pmulabs.viewModels.SharedViewModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.text.SimpleDateFormat
import java.util.Date

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ArticlePageScreen(navController: NavController,modifier: Modifier = Modifier,sharedViewModel: SharedViewModel) {
    val argument = sharedViewModel.argument.value
    val context = LocalContext.current

    var id=navController.currentBackStackEntry?.arguments?.getString(ARTICLE_ARGUMENT_KEY).toString()
    var textComm by rememberSaveable { mutableStateOf("") }
    var article by remember { mutableStateOf<Article?>(null) }
    var tag by remember { mutableStateOf<Tag?>(null) }
    var tagId = 0
    if (id.toString() != "null") {
        LaunchedEffect(Unit) {
            withContext(Dispatchers.IO) {
                NewsPortalDatabase.getInstance(context).articleDao().getArticleById(id.toInt()).collect { data ->
                    article=data
                    tagId=data.tagId
                    NewsPortalDatabase.getInstance(context).tagDao().getTagById(tagId).collect { data ->
                        tag=data
                    }
                }
            }
        }
    }

    val formatter = SimpleDateFormat("dd.MM.YY")
    val publishDate=formatter.format(Date(article?.publishDate ?: 0))

    val comms = remember { mutableStateListOf<Comment>() }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).commentDao().getAll().collect { data ->
                comms.clear()
                comms.addAll(data)
            }
        }
    }

    var getUser by remember { mutableStateOf<User?>(null) }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).userDao()
                .getUserById(argument.toString().toInt()).collect { data ->
                    getUser=data
                }
        }
    }

    LazyColumn(
        contentPadding= PaddingValues(top=105.dp, bottom = 50.dp,start=15.dp,end=15.dp),
        modifier = modifier
            .requiredWidth(width = 412.dp)
            .requiredHeight(height = 915.dp)
            .background(color = Color.White)
            .fillMaxSize(),
        verticalArrangement = Arrangement.spacedBy(10.dp)
    ) {
        item{
            BackButton(modifier=Modifier
                .clickable { navController.popBackStack()}
                .offset(
                    y=0.dp
                ))
        }
        item {
            Row(
                modifier = Modifier
                    .requiredWidth(width = 350.dp)
                    .requiredHeight(height = 33.dp)
                    .offset(
                        y=0.dp
                    )
            ) {
                Text(
                    text = "${publishDate}",
                    color = Color(0xff423a99),
                    style = MaterialTheme.typography.headlineMedium)
                Text(
                    text = "#${tag?.title}",
                    color = Color(0xff423a99),
                    style = MaterialTheme.typography.headlineMedium,
                    modifier = Modifier
                        .offset(x = 20.dp,
                            y = 0.dp))
            }
        }
        item {
            Text(
                text = "${article?.title}",
                color = Color(0xff423a99),
                style = TextStyle(
                    fontSize = 32.sp,
                    fontWeight = FontWeight.Bold),
                modifier = Modifier
                    .offset(x = 10.dp,
                        y = 0.dp)
                    .requiredWidth(width = 395.dp))

        }
        item {
            Text(
                text = "${article?.text}",
                color = Color(0xff423a99),
                style = MaterialTheme.typography.headlineMedium,
                modifier = Modifier
                    .offset(x = 5.dp,
                        y = 0.dp)
                    .requiredWidth(width = 375.dp))
        }
        item {
            Box(
                modifier = modifier
                    .requiredWidth(width = 412.dp)
                    .requiredHeight(height = 53.dp)
                    .offset(
                        y=0.dp
                    )
            ) {
                Text(
                    text = "Comments:",
                    color = Color(0xff423a99),
                    style = TextStyle(
                        fontSize = 32.sp,
                        fontWeight = FontWeight.Bold),
                    modifier = Modifier
                        .align(alignment = Alignment.TopStart)
                        .offset(x = 15.dp,
                            y = 6.dp)
                        .requiredWidth(width = 395.dp)
                        .requiredHeight(height = 43.dp))
                HorizontalDivider(
                    thickness=5.dp,
                    modifier = Modifier
                        .border(BorderStroke(3.dp, Color(0xff423a99))), color = Color(0xff423a99)
                )
                HorizontalDivider(
                    thickness=5.dp,
                    modifier = Modifier
                        .align(alignment = Alignment.TopStart)
                        .border(BorderStroke(3.dp, Color(0xff423a99)))
                        .offset(
                            y = 53.dp), color = Color(0xff423a99)
                )
            }
        }
        item{
            OutlinedTextField(
                value = textComm,
                onValueChange = {
                    textComm=it
                },
                colors = TextFieldDefaults.textFieldColors(
                    containerColor = Color.White,
                    focusedIndicatorColor = Color(0xff423a99),
                    focusedLabelColor = Color(0xff423a99),
                    unfocusedIndicatorColor = Color(0xff423a99),
                    unfocusedLabelColor = Color(0xff423a99)
                ),
                modifier = Modifier
                    .fillMaxWidth()
                    .wrapContentHeight()
                    .padding(10.dp),
                label = { Text("Comment") },
                placeholder = { Text("Write comment...") },
                minLines = 3,
                maxLines = 3,
            )
            Button(
                colors = ButtonColors(
                    containerColor = Color(0xff423a99),
                    disabledContainerColor = Color(0xff423a99),
                    contentColor = Color.White,
                    disabledContentColor = Color.White
                ),
                onClick = {
                    if(textComm.isNotEmpty()) {
                        var newComment =
                            Comment(null, textComm, getUser?.id.toString().toInt(), id.toInt())
                        CoroutineScope(Dispatchers.IO).launch {
                            NewsPortalDatabase.getInstance(context).commentDao().insert(newComment)
                        }
                        textComm = ""
                    }
                },
                modifier = Modifier
                    .padding(start = 230.dp)
                    .fillMaxWidth(0.9f)
                    .height(40.dp)
            ) {
                Text(text = "Send", fontSize = 20.sp)
            }
        }
        item{
            comms.forEach{ comm ->
                if(comm.articleId==article?.id && comm.text!="") {
                    Spacer(modifier = Modifier.padding(0.dp))
                    HorizontalDivider(
                        thickness=3.dp,
                        modifier = Modifier
                            .border(BorderStroke(3.dp, Color(0xff423a99))), color = Color(0xff423a99)
                    )
                    CommentItem(comm = comm, sharedViewModel = sharedViewModel)
                }
            }

        }
    }
}
