package com.example.pmulabs.screensMobile

import android.annotation.SuppressLint
import android.util.Log
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.DatePicker
import androidx.compose.material3.DatePickerColors
import androidx.compose.material3.DatePickerDialog
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.material3.rememberDatePickerState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableLongStateOf
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import androidx.navigation.NavHostController
import androidx.navigation.compose.rememberNavController
import com.example.pmulabs.basecomponents.navigate.BottomBar
import com.example.pmulabs.basecomponents.navigate.BottomBarScreen
import com.example.pmulabs.basecomponents.navigate.SearchAppBar
import com.example.pmulabs.basecomponents.navigate.TopBar
import com.example.pmulabs.designElem.items.ArticleItem
import com.example.pmulabs.graphs.HomeNavGraph
import com.example.pmulabs.room.database.NewsPortalDatabase
import com.example.pmulabs.room.models.Article
import com.example.pmulabs.viewModels.SearchViewModel
import com.example.pmulabs.viewModels.SearchWidget
import com.example.pmulabs.viewModels.SharedViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.text.SimpleDateFormat
import java.util.Calendar
import java.util.Date

@Composable
fun MainScreen(navController: NavController, modifier: Modifier = Modifier,sharedViewModel: SharedViewModel){

    val context = LocalContext.current
    val articles = remember { mutableStateListOf<Article>() }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).articleDao().getAll().collect { data ->
                articles.clear()
                articles.addAll(data)
            }
        }
    }
    LazyColumn(
        modifier=Modifier
            .background(Color.White)
            .fillMaxSize(),
        contentPadding = PaddingValues(top=75.dp, bottom = 70.dp, start = 10.dp,end=10.dp),
        verticalArrangement = Arrangement.spacedBy(15.dp)){
        items(items = articles){ article ->
                ArticleItem(article = article, sharedViewModel = sharedViewModel, onArticleClick = {navController.navigate(BottomBarScreen.ArticlePage.passId(article.id.toString()))})
        }
    }
}

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun LoadScreen(navController: NavHostController=rememberNavController(), searchViewModel: SearchViewModel, sharedViewModel: SharedViewModel){

    val searchWidgetState by searchViewModel.searchWidgetState
    val searchTextState by searchViewModel.searchTextState
    val formatter = SimpleDateFormat("dd-MMMM-YY")

    val calendar = Calendar.getInstance()
    calendar.set(2023, 0, 1) // add year, month (Jan), date

    // set the initial date
    val datePickerState = rememberDatePickerState(initialSelectedDateMillis = calendar.timeInMillis)

    var showDatePicker by remember {
        mutableStateOf(false)
    }

    var selectedDate by remember {
        mutableLongStateOf(calendar.timeInMillis) // or use mutableStateOf(calendar.timeInMillis)
    }

    if (showDatePicker) {
        DatePickerDialog(
            colors= DatePickerColors(
                containerColor = Color.White,
                currentYearContentColor=Color(0xff423a99),
                dateTextFieldColors = TextFieldDefaults.textFieldColors(
                    Color(0xff423a99),
                    containerColor = Color.White,
                    focusedLabelColor = Color(0xff423a99),
                    focusedPlaceholderColor = Color(0xff423a99),
                    focusedTrailingIconColor = Color(0xff423a99)
                ),
                dayContentColor=Color(0xff423a99),
                dayInSelectionRangeContainerColor=Color(0xff423a99),
                dayInSelectionRangeContentColor=Color(0xff423a99),
                disabledDayContentColor=Color(0xff423a99),
                disabledSelectedDayContainerColor=Color(0xff423a99),
                disabledSelectedDayContentColor=Color(0xff423a99),
                disabledSelectedYearContainerColor=Color(0xff423a99),
                disabledSelectedYearContentColor=Color(0xff423a99),
                disabledYearContentColor=Color(0xff423a99),
                dividerColor=Color(0xff423a99),
                headlineContentColor=Color(0xff423a99),
                navigationContentColor=Color(0xff423a99),
                selectedDayContainerColor=Color(0xff423a99),
                selectedDayContentColor=Color.White,
                selectedYearContainerColor=Color(0xff423a99),
                selectedYearContentColor=Color.White,
                subheadContentColor=Color(0xff423a99),
                titleContentColor=Color(0xff423a99),
                todayContentColor=Color.White,
                todayDateBorderColor=Color(0xff423a99),
                weekdayContentColor=Color(0xff423a99),
                yearContentColor=Color(0xff423a99)
            ),
            onDismissRequest = {
                showDatePicker = false
            },
            confirmButton = {
                TextButton(
                    modifier=Modifier
                        .background(Color.White),
                    colors = ButtonColors(
                        contentColor = Color(0xff423a99),
                        containerColor=Color.Transparent,
                        disabledContainerColor=Color.Transparent,
                        disabledContentColor= Color.Transparent
                    ),
                    onClick = {
                    showDatePicker = false
                    selectedDate = datePickerState.selectedDateMillis!!
                    Log.d("Selected date: ", formatter.format(Date(selectedDate)))
                        navController.navigate(BottomBarScreen.Calendar.passDate(formatter.format(Date(selectedDate)))){
                            popUpTo(BottomBarScreen.Main.route)
                        }
                }) {
                    Text(text = "Confirm")
                }
            },
            dismissButton = {
                TextButton(
                    modifier=Modifier
                        .background(Color.White),
                    colors = ButtonColors(
                        contentColor = Color(0xff423a99),
                        containerColor=Color.Transparent,
                        disabledContainerColor=Color.Transparent,
                        disabledContentColor= Color.Transparent
                    ),
                    onClick = {
                        showDatePicker = false
                        navController.navigate(BottomBarScreen.Main.route)
                }) {
                    Text(text = "Cancel")
                }
            }
        ) {
            DatePicker(
                modifier= Modifier
                    .background(Color.White)
                    .clip(RoundedCornerShape(15.dp)),
                colors = DatePickerColors(
                        containerColor = Color.White,
                        currentYearContentColor=Color(0xff423a99),
                        dateTextFieldColors = TextFieldDefaults.textFieldColors(
                            Color(0xff423a99),
                            containerColor = Color.White,
                            focusedLabelColor = Color(0xff423a99),
                            focusedPlaceholderColor = Color(0xff423a99),
                            focusedTrailingIconColor = Color(0xff423a99)
                        ),
                        dayContentColor=Color(0xff423a99),
                        dayInSelectionRangeContainerColor=Color(0xff423a99),
                        dayInSelectionRangeContentColor=Color(0xff423a99),
                        disabledDayContentColor=Color(0xff423a99),
                        disabledSelectedDayContainerColor=Color(0xff423a99),
                        disabledSelectedDayContentColor=Color(0xff423a99),
                        disabledSelectedYearContainerColor=Color(0xff423a99),
                        disabledSelectedYearContentColor=Color(0xff423a99),
                        disabledYearContentColor=Color(0xff423a99),
                        dividerColor=Color(0xff423a99),
                        headlineContentColor=Color(0xff423a99),
                        navigationContentColor=Color(0xff423a99),
                        selectedDayContainerColor=Color(0xff423a99),
                        selectedDayContentColor=Color.White,
                        selectedYearContainerColor=Color(0xff423a99),
                        selectedYearContentColor=Color.White,
                        subheadContentColor=Color(0xff423a99),
                        titleContentColor=Color(0xff423a99),
                        todayContentColor=Color.White,
                        todayDateBorderColor=Color(0xff423a99),
                        weekdayContentColor=Color(0xff423a99),
                        yearContentColor=Color(0xff423a99)
                ),
                state = datePickerState
            )
        }
    }

    Scaffold(
        bottomBar = { BottomBar(navController = navController) },
        topBar = { MainAppBar(searchWidgetState= searchWidgetState,
            searchTextState= searchTextState,
            onTextChange= { searchViewModel.updateSearchTextState(newValue = it)
                if(it!="") {
                    navController.navigate(BottomBarScreen.Search.passText(it))
                }},
            onCloseClicked= {
                searchViewModel.updateSearchWidgetState(newValue = SearchWidget.CLOSED)
                navController.navigate(BottomBarScreen.Main.route)
            },
            onSearchClicked= {
                navController.navigate(BottomBarScreen.Search.passText(it))
            },
            onSearchTriggered= {
                searchViewModel.updateSearchWidgetState(newValue = SearchWidget.OPENED)
            },
            onCalendarTriggered = {
                showDatePicker = true
            })}
    ) {
        Modifier
            .padding(it)
        HomeNavGraph(navController = navController,sharedViewModel)
    }
}

@Composable
fun MainAppBar(
    searchWidgetState: SearchWidget,
    searchTextState: String,
    onTextChange: (String) -> Unit,
    onCloseClicked: () -> Unit,
    onSearchClicked: (String) -> Unit,
    onSearchTriggered: () -> Unit,
    onCalendarTriggered: () -> Unit
) {
    when (searchWidgetState) {
        SearchWidget.CLOSED -> {
            TopBar(
                onSearchClicked = onSearchTriggered,
                onCalendarClicked = onCalendarTriggered
            )
        }
        SearchWidget.OPENED -> {
            SearchAppBar(
                text = searchTextState,
                onTextChange = onTextChange,
                onCloseClicked = onCloseClicked,
                onSearchClicked = onSearchClicked
            )
        }
    }
}




