package com.example.pmulabs.screensMobile.filterScreens

import android.util.Log
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import com.example.pmulabs.basecomponents.navigate.BottomBarScreen
import com.example.pmulabs.basecomponents.navigate.CALENDAR_ARGUMENT_DATE
import com.example.pmulabs.designElem.elem.BackButton
import com.example.pmulabs.designElem.items.ArticleItem
import com.example.pmulabs.room.database.NewsPortalDatabase
import com.example.pmulabs.room.models.Article
import com.example.pmulabs.viewModels.SharedViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.text.SimpleDateFormat

@Composable
fun  CalendarScreen(navController: NavController, modifier: Modifier = Modifier,sharedViewModel: SharedViewModel){
    val context = LocalContext.current
    val articles = remember { mutableStateListOf<Article>() }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            NewsPortalDatabase.getInstance(context).articleDao().getAll().collect { data ->
                articles.clear()
                articles.addAll(data)
            }
        }
    }
    var date=navController.currentBackStackEntry?.arguments?.getString(CALENDAR_ARGUMENT_DATE).toString()
    Log.d("date: ",date)
    val formatter = SimpleDateFormat("dd-MMMM-YY")

    LazyColumn(
        modifier= Modifier
            .background(Color.White)
            .fillMaxSize(),
        contentPadding = PaddingValues(top =75.dp, bottom = 10.dp, start = 10.dp,end=10.dp),
        verticalArrangement = Arrangement.spacedBy(1.dp)){
        item{
            BackButton(modifier=Modifier
                .clickable { navController.popBackStack()})
        }
        items(items = articles){ article ->
            val publishDate=formatter.format(article.publishDate)
            if(publishDate==date) {
                ArticleItem(
                    article = article,
                    sharedViewModel=sharedViewModel,
                    onArticleClick = {navController.navigate(BottomBarScreen.ArticlePage.passId(article.id.toString()))}
                )
            }
        }
    }
}